<?php

/**
 * Administrative listing of all clients.
 */
function jobtrack_admin_client_overview() {
  $rows = array();
  $header = array(
    array('data' => t('Client name'), 'field' => 'name'),
    array('data' => t('Status'), 'field' => 'status'),
    array('data' => t('From address'), 'field' => 'mailfrom'));
  $sql = 'SELECT clid, name, status, mailfrom FROM {jobtrack_client}';
  $sql .= tablesort_sql($header);
  $result = pager_query($sql, 50, 0);
  while ($client = db_fetch_object($result)) {
    $rows[] = array(
      check_plain($client->name),
      $client->status == 1 ? t('enabled') : t('disabled'),
      check_plain($client->mailfrom),
      l(t('edit'), "admin/jobtrack/clients/$client->clid/edit") .' | '.
      l(t('fetch'), "admin/jobtrack/clients/$client->clid/fetch"));
  }
  if (empty($rows)) {
    drupal_set_message('There are currently no clients defined.');
  }
  return theme('table', $header, $rows);
}

/**
 * Administratively add/update a client.
 */
function jobtrack_admin_client(&$form_state, $client = NULL) {
  $form = array();

  $form['name'] = array(
    '#title' => t('Client name'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->name) ? check_plain($client->name) : '',
    '#description' => t('The full name of your client.  This name will be used when building menus, sending emails, and anywhere else the name of your client is visibly displayed.'),
  );
  $form['path'] = array(
    '#title' => t('Path'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->path) ? check_plain($client->path) : '',
    '#description' => t("Used to build the path to this client\s tickets.  The path should only be comprised of alphanumeric characters, dashes, and underscores."),
  );
  $form['server_name'] = array(
    '#title' => t('Server name'),
    '#type' => 'textfield',
    '#description' => t('Hostname or IP address.'),
    '#required' => TRUE,
    '#default_value' => isset($client->server_name) ? check_plain($client->server_name) : '',
    '#description' => t("The hostname or IP address of the mail server where your client's dedicated email address is hosted.  For example, client %foo may receive email at %email, and so the server name may be %hostname.  The server name may also be an IP address, such as %ip.", array('%foo' => 'foo', '%email' => 'foo@support.sample.com', '%hostname' => 'support.sample.com', '%ip' => '192.168.1.1')),
  );
  $form['server_username'] = array(
    '#title' => t('Server username'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->server_username) ? check_plain($client->server_username) : '',
    '#description' => t("The client's email username.  If a client receives email at %email, the server username is often simply %user.  Jobtrack uses this username to log into the mail server to download messages.", array('%email' => 'foo@support.sample.com', '%user' => 'foo')),
  );
  $form['server_password'] = array(
    '#title' => t('Server password'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->server_password) ? check_plain($client->server_password) : '',
    '#description' => t("The client's email password.  Jobtrack uses this password to log into the mail server to download messages. %notice", array('%notice' => t('Note: this is not a password you request from your client.  This is a password you configure on your mail server for managing support tickets for a client.'))),
  );
  $form['mailbox'] = array(
    '#title' => t('Mailbox'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->mailbox) ? check_plain($client->mailbox) : 'INBOX',
    '#description' => t("The mailbox path on your mail server.  You generally do not need to modify this setting."),
  );
  $form['mailfrom'] = array(
    '#title' => t('From address'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->mailfrom) ? check_plain($client->mailfrom) : '',
    '#description' => t('The email address jobtrack should use when sending notification emails.  This must be the same client-specific email address users will send messages to when creating and updating tickets via email.  This will often be comprised of your server username and your server name, such as %email.', array('%email' => 'foo@support.sample.com')),
  );
  $form['port'] = array(
    '#title' => t('Port'),
    '#type' => 'textfield',
    '#required' => TRUE,
    '#default_value' => isset($client->port) ? check_plain($client->port) : '',
    '#description' => t('Specify the TCP port to connect to when downloading email messages.  If using pop3, this port is normally 110.  If using secure pop3, this port is normally 995.  If using imap, this port is normally 220.  If using secure imap this port is normally 993.'),
  );
  $form['protocol'] = array(
    '#type' => 'radios',
    '#title' => t('Protocol'),
    '#default_value' => 0,
    '#options' => array('pop3', 'secure pop3', 'imap', 'secure imap', 'local file'),
    '#required' => TRUE,
    '#description' => t('Select the protocol used to obtain messages from your mail server.'),
    '#default_value' => isset($client->protocol) ? $client->protocol : 0,
  );

  $form['status'] = array(
    '#title' => t('Status'),
    '#type' => 'radios',
    '#options' => array(1 => t('Enabled'), 2 => t('Disabled')),
    '#default_value' => isset($client->status) ? $client->status : 1,
  );
/*
  TODO: Implement support for this.
  $form['delete_read'] = array(
    '#title' => t('Delete messages from server after downloading'),
    '#type' => 'checkbox',
    '#default_value' => TRUE,
    '#default_value' => isset($client->delete_read) ? $client->delete_read : 1,
  );
*/
  $form['autosubscribe'] = array(
    '#title' => t('Auto-subscribe'),
    '#type' => 'textfield',
    '#default_value' => isset($client->autosubscribe) ? $client->autosubscribe : '',
    '#description' => t('Enter one or more comma-seperated usernames to auto-subscribe to !client issues', array('!client' => $client->name)),
  );
  $form['domains'] = array(
    '#title' => t('Valid email domains'),
    '#type' => 'textfield',
    '#default_value' => isset($client->domains) ? check_plain($client->domains) : '',
    '#description' => t('Enter one or more comma-seperated domains which are allowed to automatically generate new tickets.  If no domains are entered, tickets are allowed from all domains (unless you globally define allowed domains).  If you globally define allowed domains but want this client to receive emails from all domains, enter a domain of "*".'),
  );
  $form['notes'] = array(
    '#title' => t('Notes'),
    '#type' => 'textarea',
    '#default_value' => isset($client->notes) ? check_plain($client->notes) : '',
    '#description' => t('Notes are only displayed on this page when editing clients.  They are optional and are provided for administrative purposes.'),
  );
  $form['clid'] = array(
    '#type' => 'hidden',
    '#value' => $client->clid,
  );
  $form['submit'] = array(
    '#value' => isset($client->clid) ? t('Update client') : t('Add client'),
    '#type' => 'submit',
  );
  if (isset($client->clid)) {
    $form['delete'] = array(
      '#value' => t('Delete client'),
      '#type' => 'submit',
    );
    $form['cancel'] = array(
      '#value' => l(t('Cancel'), 'admin/jobtrack/clients'),
    );
  }

  return $form;
}

function jobtrack_admin_client_validate($form, &$form_state) {
  if (is_array($form['autosubscribe'] && !empty($form['autosubscribe']['#value']))) {
    $subscribe = array();
    $autosubscribe = explode(',', $form['autosubscribe']['#value']);
    foreach ($autosubscribe as $name) {
      $account = user_load(array('name' => trim($name)));
      if (!is_object($account) || !$account->uid) {
        form_set_error('autosubscribe', t('User !username does not exist.', array('!username' => $name)));
      }
    }
  }
  if (is_array($form['domains']) && !empty($form['domains']['#value'])) {
    _jobtrack_validate_domains('domains', explode(',', $form['domains']['#value']));
  }
}

/**
 * Add/update client information.
 */
function jobtrack_admin_client_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#value'] == t('Delete client')) {
    drupal_set_message(t('Client %client deleted.', array('%client' => $form_state['values']['name'])));
    db_query('DELETE FROM {jobtrack_client} WHERE clid = %d', $form_state['values']['clid']);
  }
  else {
    db_query("UPDATE {jobtrack_client} SET name = '%s', path = '%s', status = %d, server_name = '%s', server_username = '%s', server_password = '%s', mailbox = '%s', mailfrom = '%s', protocol = %d, port = %d, delete_read = %d, autosubscribe = '%s', domains = '%s', notes = '%s' WHERE clid = %d", $form_state['values']['name'], $form_state['values']['path'], $form_state['values']['status'], $form_state['values']['server_name'], $form_state['values']['server_username'], $form_state['values']['server_password'], $form_state['values']['mailbox'], $form_state['values']['mailfrom'], $form_state['values']['protocol'], $form_state['values']['port'], $form_state['values']['delete_read'], $form_state['values']['autosubscribe'], $form_state['values']['domains'], $form_state['values']['notes'], $form_state['values']['clid']);
    if (!db_affected_rows()) {
      drupal_set_message(t('Client %client added.', array('%client' => $form_state['values']['name'])));
      db_query("INSERT INTO {jobtrack_client} (name, path, status, server_name, server_username, server_password, mailbox, mailfrom, protocol, port, delete_read, autosubscribe, domains, notes) VALUES('%s', '%s', %d, '%s', '%s', '%s', '%s', '%s', %d, %d, %d, '%s', '%s', '%s')", $form_state['values']['name'], $form_state['values']['path'], $form_state['values']['status'], $form_state['values']['server_name'], $form_state['values']['server_username'], $form_state['values']['server_password'], $form_state['values']['mailbox'], $form_state['values']['mailfrom'], $form_state['values']['protocol'], $form_state['values']['port'], $form_state['values']['delete_read'], $form_state['values']['autosubscribe'], $form_state['values']['domains'], $form_state['values']['notes']);
    }
    else {
      drupal_set_message(t('Client %client updated.', array('%client' => $form_state['values']['name'])));
    }
  }
  // clean page cache and block cache
  cache_clear_all();
  // rebuild the menu
  menu_rebuild();
  drupal_goto('admin/jobtrack/clients');
}

/**
 * Admin page block.
 */
function jobtrack_admin_menu_block_page() {
  $item = menu_get_item();
  if ($content = system_admin_menu_block($item)) {
    $output = theme('admin_block_content', $content);
  }
  else {
    $output = t('You do not have any administrative items.');
  }
  return $output;
}


function jobtrack_admin_settings() {
  $form = array();

  $form['jobtrack_global_domains'] = array(
    '#type' => 'textfield',
    '#title' => t('Globally allowed domains'),
    '#description' => t('Optionally enter one or more domains which are allowed to automatically create tickets by sending an email.  If no domains are entered, or you specify "*", all domains are allowed.  If entering multiple domains, separate them by commas.  These domains will be allowed for all clients.'),
    '#default_value' => check_plain(variable_get('jobtrack_global_domains', '')),
  );

  return system_settings_form($form);
}

/**
 * Be sure valid domains are entered.
 */
function jobtrack_admin_settings_validate($form, &$form_state) {
  if (!empty($form['jobtrack_global_domains']['#value'])) {
    _jobtrack_validate_domains('jobtrack_global_domains', explode(',', trim($form['jobtrack_global_domains']['#value'])));
  }
}

/**
 * Validate domains by doing a dns lookup on each.
 */
function _jobtrack_validate_domains($field, $domains = array()) {
  foreach ($domains as $domain) {
    $domain = trim($domain);
    if ($domain && ($domain != '*' && !checkdnsrr($domain))) {
      form_set_error($field, t('Invalid domain: %domain', array('%domain' => $domain)));
    }
  }
}
